import { forwardRef } from "react";
// @ts-ignore
import { motion, AnimatePresence } from "framer-motion/dist/framer-motion";
import { color } from "styled-system";
import styled from "styled-components";
import { isNumber } from "../utils/type-util";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const BadgeCount = styled.sup.withConfig({
  displayName: "badge__BadgeCount",
  componentId: "sc-nc1i9z-0"
})(["--badge-height:20px;position:absolute;top:0;right:0;transform:translate(50%,-50%);box-shadow:0 0 0 1px ", ";background:", ";color:#fff;z-index:auto;&[data-dot=\"true\"]{width:8px;height:8px;border-radius:100%;}&[data-dot=\"false\"]{padding:0 6px;border-radius:calc(var(--badge-height) / 2);min-width:var(--badge-height);height:var(--badge-height);line-height:var(--badge-height);font-size:", "px;font-weight:normal;white-space:nowrap;text-align:center;}&[data-size=\"sm\"]{--badge-height:15px;padding:0px 4px;font-size:10px;}", ""], p => p.theme.colors.bg.base, p => p.theme.colors.primary.base, p => p.theme.fontSizes.sm, color);
const BadgeMotion = styled(motion.div).withConfig({
  displayName: "badge__BadgeMotion",
  componentId: "sc-nc1i9z-1"
})(["&[data-wrapper=\"true\"]{position:absolute;top:0;right:0;transform-origin:100% 0;}&[data-wrapper=\"false\"] ", "{transform:none;position:relative;top:auto;display:block;}"], BadgeCount);
const BadgeRoot = styled.span.withConfig({
  displayName: "badge__BadgeRoot",
  componentId: "sc-nc1i9z-2"
})(["position:relative;display:inline-block;color:unset;line-height:1;"]);
function stringifyCount(count) {
  let overflowCount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  if (!isNumber(count)) {
    return "";
  }
  if (count > overflowCount) {
    return `${overflowCount}+`;
  }
  return String(count);
}
export const Badge = /*#__PURE__*/forwardRef((_ref, ref) => {
  let {
    count,
    overflowCount = 99,
    dot = false,
    size = "md",
    offset,
    color,
    bg,
    children,
    ...rest
  } = _ref;
  const countString = stringifyCount(count, overflowCount);
  const countDisplay = dot ? "" : countString;
  const title = isNumber(count) ? String(count) : undefined;
  const show = dot || countString.length > 0 && countString !== "0";
  const offsetStyle = offset ? {
    right: -offset[0],
    top: offset[1]
  } : {};
  return /*#__PURE__*/_jsxs(BadgeRoot, {
    ref: ref,
    ...rest,
    children: [children, /*#__PURE__*/_jsx(AnimatePresence, {
      children: show ? /*#__PURE__*/_jsx(BadgeMotion, {
        initial: {
          opacity: 0,
          scale: 0
        },
        animate: {
          opacity: 1,
          scale: 1
        },
        exit: {
          opacity: 0,
          scale: 0
        },
        transition: {
          type: "spring",
          damping: 20,
          stiffness: 300
        },
        "data-wrapper": !!children,
        children: /*#__PURE__*/_jsx(BadgeCount, {
          color: color,
          bg: bg,
          title: title,
          "data-size": size,
          "data-dot": dot,
          style: offsetStyle,
          children: countDisplay
        })
      }) : null
    })]
  });
});
if (process.env.NODE_ENV !== "production") {
  Badge.displayName = "Badge";
}