import { forwardRef } from "react";
import { useDisclosure, useControllableState, useForkHandler } from "../hooks";
import { ChevronDownIcon } from "../icons";
import { KeyCode } from "../utils/key-code";
import { Input } from "./input";
import { Grid } from "./grid";
import { Flex } from "./flex";
import { Checkbox } from "./checkbox";
import { Dropdown } from "./dropdown";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
function getBit(i, num) {
  return num & 1 << i;
}
function setBit(i, num, val) {
  return num ^ (-val ^ num) & 1 << i;
}
export const BinaryInput = /*#__PURE__*/forwardRef((_ref, ref) => {
  let {
    value,
    defaultValue = 0,
    onChange,
    onOpen,
    onClose,
    onBlur,
    onKeyDown,
    bits = 16,
    max = 16,
    disabled,
    className,
    ...rest
  } = _ref;
  const {
    isOpen,
    open,
    close
  } = useDisclosure({
    onOpen,
    onClose
  });
  const [decimal, setDecimal] = useControllableState(value, defaultValue);
  const bitArray = [...Array(bits).keys()];
  const activeBits = bitArray.filter(i => getBit(i, decimal) !== 0).map(i => i + 1);
  const handleBlur = () => {
    close();
  };
  const handleKeyDown = event => {
    if (event.key === KeyCode.ENTER) {
      open();
    } else if (event.key === KeyCode.ESC) {
      event.stopPropagation();
      close();
    }
  };
  const handleSelect = (i, val) => {
    const next = setBit(i, decimal, val);
    setDecimal(next);
    onChange?.(next);
  };
  const inputProps = {
    onBlur: useForkHandler(onBlur, handleBlur),
    onKeyDown: useForkHandler(onKeyDown, handleKeyDown),
    disabled,
    ...rest
  };
  const overlay = /*#__PURE__*/_jsx(Grid, {
    columns: 4,
    gap: 1,
    children: bitArray.map(i => {
      const checked = getBit(i, decimal) !== 0;
      return /*#__PURE__*/_jsx(Flex, {
        p: 1,
        justify: "center",
        children: /*#__PURE__*/_jsx(Checkbox, {
          checked: checked,
          onChange: val => handleSelect(i, val),
          disabled: activeBits.length >= max && !checked,
          orientation: "vertical",
          children: i + 1
        })
      }, i);
    })
  });
  return /*#__PURE__*/_jsx(Dropdown, {
    overlay: overlay,
    isOpen: isOpen,
    onOpen: open,
    onClose: close,
    disabled: disabled,
    children: /*#__PURE__*/_jsxs(Input.Group, {
      className: className,
      children: [/*#__PURE__*/_jsx(Input, {
        ref: ref,
        value: activeBits.join(),
        role: "combobox",
        cursor: "pointer",
        readOnly: true,
        ...inputProps
      }), /*#__PURE__*/_jsx(Input.Suffix, {
        children: /*#__PURE__*/_jsx(ChevronDownIcon, {})
      })]
    })
  });
});
if (process.env.NODE_ENV !== "production") {
  BinaryInput.displayName = "BinaryInput";
}