import { cloneElement, useRef, useEffect } from "react";
import { useForkRef, useCallbackRef } from "../hooks";
import { ownerDocument } from "../utils/dom-util";
import { Fragment as _Fragment } from "react/jsx-runtime";
import { jsx as _jsx } from "react/jsx-runtime";
function mapEventPropToEvent(eventProp) {
  return eventProp.substring(2).toLowerCase();
}
function clickedRootScrollbar(event, doc) {
  return doc.documentElement.clientWidth < event.clientX || doc.documentElement.clientHeight < event.clientY;
}
function isInsideDom(event, doc, el) {
  if (event.composedPath) {
    return event.composedPath().indexOf(el) > -1;
  }
  return !doc.documentElement.contains(event.target) || el.contains(event.target);
}
export function ClickAwayListener(props) {
  const {
    children,
    disableReactTree = false,
    mouseEvent = "onClick",
    onClickAway
  } = props;
  const nodeRef = useRef(null);
  const activatedRef = useRef(false);
  const syntheticEventRef = useRef(false);
  const handleRef = useForkRef(
  // @ts-expect-error
  children.ref, nodeRef);
  useEffect(() => {
    // Ensure that this component is not "activated" synchronously.
    // https://github.com/facebook/react/issues/20074
    setTimeout(() => {
      activatedRef.current = true;
    }, 0);
    return () => {
      activatedRef.current = false;
    };
  }, []);
  const handleClickAway = useCallbackRef(event => {
    // Given developers can stop the propagation of the synthetic event,
    // we can only be confident with a positive value.
    const insideReactTree = syntheticEventRef.current;
    syntheticEventRef.current = false;
    const doc = ownerDocument(nodeRef.current);

    // 1. IE11 support, which trigger the handleClickAway even after the unbind
    // 2. The child might render null.
    // 3. Behave like a blur listener.
    if (!activatedRef.current || !nodeRef.current || "clientX" in event && clickedRootScrollbar(event, doc)) {
      return;
    }
    if (!isInsideDom(event, doc, nodeRef.current) && (disableReactTree || !insideReactTree)) {
      onClickAway?.(event);
    }
  });
  useEffect(() => {
    if (mouseEvent !== false) {
      const eventName = mapEventPropToEvent(mouseEvent);
      const doc = ownerDocument(nodeRef.current);
      doc.addEventListener(eventName, handleClickAway);
      return () => {
        doc.removeEventListener(eventName, handleClickAway);
      };
    }
    return;
  }, [handleClickAway, mouseEvent]);
  const childrenProps = {
    ref: handleRef
  };
  if (mouseEvent !== false) {
    // Keep track of mouse events that bubbled up through the portal
    childrenProps[mouseEvent] = event => {
      syntheticEventRef.current = true;
      children.props[mouseEvent]?.(event);
    };
  }
  return /*#__PURE__*/_jsx(_Fragment, {
    children: /*#__PURE__*/cloneElement(children, childrenProps)
  });
}
if (process.env.NODE_ENV !== "production") {
  ClickAwayListener.displayName = "ClickAwayListener";
}