import { forwardRef, createContext, useContext, useMemo } from "react";
import styled from "styled-components";
import { useControllableState, useCallbackRef } from "../hooks";
import { Flex } from "./flex";
import { Box } from "./box";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const Input = styled.input.withConfig({
  displayName: "radio__Input",
  componentId: "sc-1hbf8qv-0"
})(["position:absolute;top:0;right:0;bottom:0;left:0;width:100%;height:100%;opacity:0;z-index:1;cursor:pointer;"]);
const Control = styled.span.withConfig({
  displayName: "radio__Control",
  componentId: "sc-1hbf8qv-1"
})(["display:inline-flex;flex:none;position:relative;top:0;left:0;width:14px;height:14px;background:", ";border:", ";border-radius:50%;transition:all 0.2s;&::after{position:absolute;top:50%;left:50%;display:table;width:14px;height:14px;margin-top:-7px;margin-left:-7px;background-color:", ";border-top:0;border-left:0;border-radius:14px;transform:scale(0);opacity:0;transition:all 0.2s ", ",opacity 0.1s;content:\" \";}"], p => p.theme.colors.gray[1], p => p.theme.borders.base, p => p.theme.colors.primary.base, p => p.theme.transitions.easeOutBack);
const Label = styled.label.withConfig({
  displayName: "radio__Label",
  componentId: "sc-1hbf8qv-2"
})(["display:inline-flex;align-items:center;vertical-align:top;position:relative;user-select:none;cursor:pointer;&:hover ", "{border-color:", ";}", ":focus + ", "{box-shadow:0 0 0 2px ", ";}&[data-state=\"checked\"]{", "{border-color:", ";&::after{transform:scale(0.5);opacity:1;}}&:hover ", "{border-color:", ";&::after{background-color:", ";}}}&[data-invalid]{", "{border-color:", ";&::after{background-color:", ";}}&:hover ", "{border-color:", ";&::after{background-color:", ";}}", ":focus:enabled + ", "{box-shadow:0 0 0 2px ", ";}}&[data-disabled]{color:", ";&,", "{cursor:not-allowed;}", "{opacity:0.5;border-color:", " !important;&::after{background:", " !important;}}&[data-state=\"checked\"] ", "{opacity:1;}}"], Control, p => p.theme.colors.gray[5], Input, Control, p => p.theme.colors.primary[2], Control, p => p.theme.colors.primary.base, Control, p => p.theme.colors.primary.hover, p => p.theme.colors.primary.hover, Control, p => p.theme.colors.danger.base, p => p.theme.colors.danger.base, Control, p => p.theme.colors.danger.hover, p => p.theme.colors.danger.hover, Input, Control, p => p.theme.colors.danger[2], p => p.theme.colors.disabled, Input, Control, p => p.theme.colors.border.base, p => p.theme.colors.disabled, Control);
const RadioGroupContext = /*#__PURE__*/createContext(undefined);
const RadioGroup = /*#__PURE__*/forwardRef((props, ref) => {
  const {
    value: valueProp,
    defaultValue = "",
    onChange,
    name,
    disabled,
    isInvalid,
    color,
    children,
    ...rest
  } = props;
  const [value, setValue] = useControllableState(valueProp, defaultValue);
  const handleChange = useCallbackRef((val, event) => {
    setValue(val);
    onChange?.(val, event);
  });
  const group = useMemo(() => ({
    value,
    onChange: handleChange,
    name,
    disabled,
    isInvalid
  }), [value, handleChange, name, disabled, isInvalid]);
  return /*#__PURE__*/_jsx(RadioGroupContext.Provider, {
    value: group,
    children: /*#__PURE__*/_jsx(Flex, {
      ref: ref,
      position: "relative",
      gap: 3,
      width: 1,
      color: color,
      role: "radiogroup",
      ...rest,
      children: children
    })
  });
});
const Radio = /*#__PURE__*/forwardRef((props, ref) => {
  const {
    value,
    onChange,
    disabled,
    className,
    children,
    ...rest
  } = props;
  const group = useContext(RadioGroupContext);
  const isChecked = group?.value === value;
  const isDisabled = group?.disabled || disabled;
  const isInvalid = group?.isInvalid;
  const handleChange = event => {
    group?.onChange?.(value, event);
    onChange?.(value, event);
  };
  return /*#__PURE__*/_jsxs(Label, {
    className: className,
    "data-state": isChecked ? "checked" : "unchecked",
    "data-disabled": isDisabled ? "" : null,
    "data-invalid": isInvalid ? "" : null,
    children: [/*#__PURE__*/_jsx(Input, {
      ref: ref,
      type: "radio",
      name: group?.name,
      value: value,
      disabled: isDisabled,
      onChange: handleChange,
      ...rest
    }), /*#__PURE__*/_jsx(Control, {}), children ? /*#__PURE__*/_jsx(Box, {
      pl: 2,
      children: children
    }) : null]
  });
});
const CompoundRadio = Object.assign(Radio, {
  id: "Radio",
  Group: RadioGroup
});
export { CompoundRadio as Radio };
if (process.env.NODE_ENV !== "production") {
  Radio.displayName = "Radio";
  RadioGroup.displayName = "RadioGroup";
}