import { forwardRef } from "react";
import styled, { css } from "styled-components";
import { useDisclosure, useControllableState, useForkHandler } from "../hooks";
import { ChevronDownIcon } from "../icons";
import { KeyCode } from "../utils/key-code";
import { Input } from "./input";
import { Dropdown } from "./dropdown";
import { ScrollBox } from "./scroll-box";
import { Flex } from "./flex";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const Option = styled.div.withConfig({
  displayName: "select__Option",
  componentId: "sc-1666d02-0"
})(["padding:4px;flex:auto;overflow:hidden;white-space:nowrap;text-overflow:ellipsis;cursor:pointer;&:hover,&:active{color:", ";background:", ";}", ""], p => p.theme.colors.gray[9], p => p.theme.colors.alpha[1], p => p.selected && css(["&,&:hover{color:", ";background:", ";}"], p.theme.colors.primary.base, p.theme.colors.gray[1]));
export const Select = /*#__PURE__*/forwardRef((_ref, ref) => {
  let {
    options = [],
    value,
    defaultValue,
    onChange,
    onOpen,
    onClose,
    onBlur,
    onKeyDown,
    disabled,
    className,
    ...rest
  } = _ref;
  const {
    isOpen,
    open,
    close
  } = useDisclosure({
    onOpen,
    onClose
  });
  const [selectedValue, setSelectedValue] = useControllableState(value, defaultValue);
  const selectedOption = options.find(opt => opt.value === selectedValue);
  const handleBlur = () => {
    close();
  };
  const handleKeyDown = event => {
    if (event.key === KeyCode.ENTER) {
      open();
    } else if (event.key === KeyCode.ESC) {
      event.stopPropagation();
      close();
    }
  };
  const handleSelect = val => {
    setSelectedValue(val);
    onChange?.(val);
    close();
  };
  const inputProps = {
    onBlur: useForkHandler(onBlur, handleBlur),
    onKeyDown: useForkHandler(onKeyDown, handleKeyDown),
    disabled,
    ...rest
  };
  const overlay = /*#__PURE__*/_jsx(ScrollBox, {
    maxHeight: 256,
    children: options.length > 0 ? options.map(opt => /*#__PURE__*/_jsx(Option, {
      onClick: () => handleSelect(opt.value),
      selected: selectedOption?.value === opt.value,
      children: opt.children ?? opt.label
    }, opt.value)) : /*#__PURE__*/_jsx(Flex, {
      p: 1,
      align: "center",
      justify: "center",
      color: "placeholder",
      children: "No Data"
    })
  });
  return /*#__PURE__*/_jsx(Dropdown, {
    overlay: overlay,
    isOpen: isOpen,
    onOpen: open,
    onClose: close,
    disabled: disabled,
    children: /*#__PURE__*/_jsxs(Input.Group, {
      className: className,
      children: [/*#__PURE__*/_jsx(Input, {
        ref: ref,
        value: selectedOption?.label ?? "",
        role: "combobox",
        cursor: "pointer",
        readOnly: true,
        ...inputProps
      }), /*#__PURE__*/_jsx(Input.Suffix, {
        children: /*#__PURE__*/_jsx(ChevronDownIcon, {})
      })]
    })
  });
});
if (process.env.NODE_ENV !== "production") {
  Select.displayName = "Select";
}